/*
 * Copyright (c) 2018 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef IDLC_C_GEN_C_HEADER_GEN_BASE_H_
#define IDLC_C_GEN_C_HEADER_GEN_BASE_H_

#include <memory>
#include <string>
#include <map>

#include "idlc/c_gen/c_gen_base.h"

namespace tidl {

class CHeaderGeneratorBase : public CGeneratorBase {
 public:
  explicit CHeaderGeneratorBase(std::shared_ptr<Document> doc);
  virtual ~CHeaderGeneratorBase() = default;

  void GenPragmaOnce(std::ofstream& stream);
  void GenExplicitLinkageOpen(std::ofstream& stream);
  void GenExplicitLinkageClose(std::ofstream& stream);
  void GenStructures(std::ofstream& stream);

 private:
  void GenStructure(std::ofstream& stream, const Structure& st);
  void GenStructureDeclaration(std::ofstream& stream, const Structure& st);
  void GenStructureParcelTo(std::ofstream& stream, const Structure& st);
  void GenStructureParcelFrom(std::ofstream& stream, const Structure& st);
  void GenStructureConstructor(std::ofstream& stream, const Structure& st);
  void GenStructureDestructor(std::ofstream& stream, const Structure& st);
  void GenStructureSetter(std::ofstream& stream, const Structure& st);
  void GenStructureGetter(std::ofstream& stream, const Structure& st);
  void GenStructureIterator(std::ofstream& stream, const Structure& st);
  void GenStructureCloner(std::ofstream& stream, const Structure& st);
};

}  // namespace tidl

#endif  // IDLC_C_GEN_C_HEADER_GEN_BASE_H_
