/*
 * Copyright (c) 2017 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef IDLC_C_GEN_C_STUB_BODY_GEN_H_
#define IDLC_C_GEN_C_STUB_BODY_GEN_H_

#include <memory>
#include <string>

#include "idlc/c_gen/c_body_gen_base.h"

namespace tidl {

class CStubBodyGen : public CBodyGeneratorBase {
 public:
  explicit CStubBodyGen(std::shared_ptr<Document> doc);
  virtual ~CStubBodyGen() = default;

  void OnInitGen(std::ofstream& stream) override;
  void OnFiniGen(std::ofstream& stream) override;

 private:
  void GenInterfaces(std::ofstream& stream);
  void GenInterface(std::ofstream& stream, const Interface& inf);
  void GenTypedefStubMethod(std::ofstream& stream);
  void GenInterfaceMethods(std::ofstream& stream, const Interface& inf);
  void GenInterfaceMethodTable(std::ofstream& stream, const Interface& inf);
  void GenInterfaceOnConnectedEventCB(std::ofstream& stream,
                                      const Interface& inf);
  void GenInterfaceOnDisconnectedEventCB(std::ofstream& stream,
                                         const Interface& inf);
  void GenInterfaceOnReceivedEventCB(std::ofstream& stream,
                                     const Interface& inf);
  void GenInterfaceRegister(std::ofstream& stream, const Interface& inf);
  void GenInterfaceUnregister(std::ofstream& stream, const Interface& inf);
  void GenInterfaceGlobalVariables(std::ofstream& stream, const Interface& inf);
  void GenInterfaceDelegators(std::ofstream& stream, const Interface& inf);
  void GenInterfaceContext(std::ofstream& stream, const Interface& inf);
  void GenInterfaceAddPrivileges(std::ofstream& stream, const Interface& inf);

 private:
  void GenInterfaceContextDeclaration(std::ofstream& stream,
                                      const Interface& inf);
  void GenInterfaceContextConstructor(std::ofstream& stream,
                                      const Interface& inf);
  void GenInterfaceContextDestructor(std::ofstream& stream,
                                     const Interface& inf);
  void GenInterfaceContextFinder(std::ofstream& stream,
                                 const Interface& inf);
  void GenInterfaceContextTagSetter(std::ofstream& stream,
                                    const Interface& inf);
  void GenInterfaceContextTagGetter(std::ofstream& stream,
                                    const Interface& inf);
  void GenInterfaceContextSenderGetter(std::ofstream& stream,
                                       const Interface& inf);

 private:
  void GenInterfaceDelegator(std::ofstream& stream, const std::string& id,
                             const Declaration& decl);
  void GenInterfaceDelegatorDeclaration(std::ofstream& stream,
                                        const std::string& id,
                                       const Declaration& decl);
  void GenInterfaceDelegatorConstructor(std::ofstream& stream,
                                      const std::string& id,
                                       const Declaration& decl);
  void GenInterfaceDelegatorDestructor(std::ofstream& stream,
                                      const std::string& id,
                                      const Declaration& decl);
  void GenInterfaceDelegatorSerializer(std::ofstream& stream,
                                      const std::string& id,
                                      const Declaration& decl);
  void GenInterfaceDelegatorDeserializer(std::ofstream& stream,
                                        const std::string& id,
                                        const Declaration& decl);
  void GenInterfaceDelegatorCloner(std::ofstream& stream,
                                  const std::string& id,
                                  const Declaration& decl);
  void GenInterfaceDelegatorInvoker(std::ofstream& stream,
                                   const std::string& id,
                                   const Declaration& decl);
  void GenInterfaceDelegatorPortSetter(std::ofstream& stream,
                                       const std::string& id,
                                       const Declaration& decl);

 private:
  std::string GetMethodString(const Interface& id, const Declaration& decl);
  std::string GetAddEventCBString(const std::string& id);
  std::string GetAddPrivilegeString(const std::string& id,
                                    const Attribute& attr);
  std::string GetTrustedModeString(const std::string& id,
                                   const Attribute& attr);
};

}  // namespace tidl

#endif  // IDLC_C_GEN_C_STUB_BODY_GEN_H_
