/*
 * Copyright (c) 2018 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef IDLC_CPP_GEN_CPP_PROXY_BODY_GEN_CB_H_
#define IDLC_CPP_GEN_CPP_PROXY_BODY_GEN_CB_H_

const char CB_DTOR[] =
R"__cpp_cb(
$$::~$$() {
  if (proxy_)
    rpc_port_proxy_destroy(proxy_);
}
)__cpp_cb";

const char CB_INVOCATION_PRE[] =
R"__cpp_cb(  if (port_ == nullptr) {
    _E("Not connected");
    throw NotConnectedSocketException();
  }

  rpc_port_parcel_h p;
  rpc_port_parcel_create(&p);
)__cpp_cb";

const char CB_INVOCATION_MID[] =
R"__cpp_cb(
    // Send
    rpc_port_parcel_send(p, port_);
)__cpp_cb";

const char CB_INVOCATION_RECEIVE[] =
R"__cpp_cb(
  if (parcel_received == nullptr) {
    _E("Invalid protocol");
    throw InvalidProtocolException();
  }
)__cpp_cb";

const char CB_INVOCATION_END[] =
R"__cpp_cb(
  rpc_port_parcel_destroy(p);
  rpc_port_parcel_destroy(parcel_received);

  return ret;
)__cpp_cb";

const char CB_PROXY_HELPER_METHODS[] =
R"__cpp_cb(
void ##::Connect() {
  int ret = rpc_port_proxy_connect(proxy_, target_appid_.c_str(), "##");
  if (ret != RPC_PORT_ERROR_NONE) {
    _E("Failed to connect ##");
    switch (ret) {
      case RPC_PORT_ERROR_INVALID_PARAMETER:
        throw InvalidIDException();

      case RPC_PORT_ERROR_IO_ERROR:
        throw InvalidIOException();

      case RPC_PORT_ERROR_PERMISSION_DENIED:
        throw PermissionDeniedException();
    }
  }
}

void ##::DisposeCallback(const std::string& tag) {
  for (auto& i : delegate_list_) {
    if (i->GetTag() == tag) {
      delegate_list_.remove(i);
      return;
    }
  }
}

void ##::ProcessReceivedEvent(rpc_port_parcel_h parcel) {
  int id = 0;
  int seq_id = 0;
  bool once = false;

  rpc_port_parcel_read_int32(parcel, &id);
  rpc_port_parcel_read_int32(parcel, &seq_id);
  rpc_port_parcel_read_bool(parcel, &once);

  for (auto& i : delegate_list_) {
    if (i->GetId() == id && i->GetSeqId() == seq_id) {
      i->OnReceivedEvent(parcel);
      if (i->IsOnce())
        delegate_list_.remove(i);
      break;
    }
  }
}

void ##::ConsumeCommand(rpc_port_parcel_h* parcel, rpc_port_h port) {
  do {
    rpc_port_parcel_h p;
    int ret = rpc_port_parcel_create_from_port(&p, port);
    int cmd;

    if (ret != 0)
      break;
    rpc_port_parcel_read_int32(p, &cmd);
    if (cmd == static_cast<int>(MethodId::__Result)) {
      *parcel = p;
      return;
    }

    rpc_port_parcel_destroy(p);
    *parcel = nullptr;
  } while (true);
  *parcel = nullptr;
}

void ##::OnConnectedCB(const char *ep, const char *port_name, rpc_port_h port, void *data) {
  ##* l = static_cast<##*>(data);
  rpc_port_h cb_port;

  l->port_ = port;
  rpc_port_proxy_get_port(l->proxy_, RPC_PORT_PORT_CALLBACK, &cb_port);
  l->callback_port_ = cb_port;
  l->listener_->OnConnected();
}

void ##::OnDisconnectedCB(const char *ep, const char *port_name, void *data) {
  ##* l = static_cast<##*>(data);
  l->delegate_list_.clear();
  l->listener_->OnDisconnected();
}

void ##::OnRejectedCB(const char *ep, const char *port_name, void *data) {
  ##* l = static_cast<##*>(data);
  l->listener_->OnRejected();
}

void ##::OnReceivedCB(const char *ep, const char *port_name, void *data) {
  ##* l = static_cast<##*>(data);
  int cmd;
  rpc_port_parcel_h parcel_received;

  if (rpc_port_parcel_create_from_port(&parcel_received, l->callback_port_) != 0) {
    _E("Failed to create parcel from port");
    return;
  }

  rpc_port_parcel_read_int32(parcel_received, &cmd);
  if (cmd != static_cast<int>(MethodId::__Callback)) {
    rpc_port_parcel_destroy(parcel_received);
    return;
  }

  l->ProcessReceivedEvent(parcel_received);
  rpc_port_parcel_destroy(parcel_received);
}
)__cpp_cb";

const char CB_PROXY_INTERFACE_CTOR[] =
R"__cpp_cb(
$$::$$(IEventListener* listener, const std::string& target_appid)
    : port_(nullptr), proxy_(nullptr), listener_(listener),
      target_appid_(target_appid) {
  int r = rpc_port_proxy_create(&proxy_);

  if (r != RPC_PORT_ERROR_NONE) {
    _E("Failed to create proxy");
    throw InvalidIOException();
  }

  rpc_port_proxy_add_connected_event_cb(proxy_, OnConnectedCB, this);
  rpc_port_proxy_add_disconnected_event_cb(proxy_, OnDisconnectedCB, this);
  rpc_port_proxy_add_rejected_event_cb(proxy_, OnRejectedCB, this);
  rpc_port_proxy_add_received_event_cb(proxy_, OnReceivedCB, this);
}
)__cpp_cb";

#endif  // IDLC_CPP_GEN_CPP_PROXY_BODY_GEN_CB_H_
