/*
 * Copyright (c) 2018 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef IDLC_CS_GEN_CS_STUB_GEN_CB_H_
#define IDLC_CS_GEN_CS_STUB_GEN_CB_H_

const char CB_DATA_MEMBERS[] =
R"__cs_cb(            private List<ServiceBase> _services = new List<ServiceBase>();
            private Type _serviceType;
)__cs_cb";

const char CB_SERVICE_BASE_FRONT[] =
R"__cs_cb(
            public abstract class ServiceBase
            {
                /// <summary>
                /// The client app ID
                /// </summary>
                public string Sender
                {
                    get; internal set;
                }

                /// <summary>
                /// The client instance ID
                /// </summary>
                public string Instance
                {
                    get; internal set;
                }

                protected ServiceBase()
                {
                }

                /// <summary>
                /// This method will be called when the client is connected
                /// </summary>
                public abstract void OnCreate();

                /// <summary>
                /// This method will be called when the client is disconnected
                /// </summary>
                public abstract void OnTerminate();
)__cs_cb";

const char CB_ON_RECEIVED_EVENT_FRONT[] =
R"__cs_cb(
            protected override bool OnReceivedEvent(string sender, string instance, Port port)
            {
                using (var p = new Parcel(port))
                {
                    ServiceBase b = null;

                    foreach (var i in  _services)
                    {
                        if (i.Instance.Equals(instance))
                        {
                            b = i;
                            break;
                        }
                    }

                    if (b == null)
                    {
                        return false;
                    }

                    using (var result = new Parcel())
                    {
                        int cmd = p.ReadInt();

                        switch ((MethodId)cmd)
                        {
)__cs_cb";

const char CB_ON_RECEIVED_EVENT_BACK[] =
R"__cs_cb(
                            default:
                                return false;
                        }
                    }

                    return true;
                }
            }
)__cs_cb";

const char CB_ON_CONNECTED_EVENT[] =
R"__cs_cb(
            protected override void OnConnectedEvent(string sender, string instance)
            {
                ServiceBase s = Activator.CreateInstance(_serviceType) as ServiceBase;
                s.Sender = sender;
                s.Instance = instance;
                s.OnCreate();
                _services.Add(s);
            }
)__cs_cb";

const char CB_ON_DISCONNECTED_EVENT[] =
R"__cs_cb(
            protected override void OnDisconnectedEvent(string sender, string instance)
            {
                foreach (var i in _services)
                {
                    if (i.Instance.Equals(instance))
                    {
                        i.OnTerminate();
                        _services.Remove(i);
                        break;
                    }
                }
            }
)__cs_cb";

const char CB_CTOR_FRONT[] =
R"__cs_cb(
            public $$() : base("$$")
            {
)__cs_cb";

const char CB_COMMON_METHODS[] =
R"__cs_cb(
            /// <summary>
            /// Listens to client apps
            /// </summary>
            /// <param name="serviceType">The type object for making service instances</param>
            /// <exception cref="InvalidIOException">
            /// Thrown when internal I/O error happen.
            /// </exception>
            /// <exception cref="ArgumentException">
            /// Thrown when serviceType is invalid.
            /// </exception>
            public void Listen(Type serviceType)
            {
                if (!typeof(ServiceBase).IsAssignableFrom(serviceType))
                    throw new ArgumentException("Invalid type");
                _serviceType = serviceType;
                Listen();
            }

            /// <summary>
            /// Gets service objects which are connected
            /// </summary>
            /// <returns>The enumerable service objects which are connected</returns>
            public IEnumerable<ServiceBase> GetServices()
            {
                return _services;
            }
)__cs_cb";

const char CB_DISPOSABLE[] =
R"__cs_cb(
            protected override void OnTerminatedEvent()
            {
                foreach (var i in _services)
                {
                    i.OnTerminate();
                }
            }
)__cs_cb";

#endif  // IDLC_CS_GEN_CS_STUB_GEN_CB_H_
