/*
 * Copyright (c) 2017 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef IDLC_PARAMETER_H_
#define IDLC_PARAMETER_H_

#include <string>
#include <list>
#include <memory>

#include "idlc/type.h"

namespace tidl {

class Parameter {
 public:
  Parameter(std::string id, ParameterType* type, unsigned line);

  const std::string& GetID() const;
  const ParameterType& GetParameterType() const;
  const unsigned GetLine() const;

 private:
  std::unique_ptr<ParameterType> type_;
  std::string id_;
  unsigned line_;
};

class Parameters {
 public:
  void Add(Parameter* param);
  const std::list<std::unique_ptr<Parameter>>& GetParams() const;
  bool Exist(Parameter* param) const;

 private:
  std::list<std::unique_ptr<Parameter>> params_;
};

}  // namespace tidl

#endif  // IDLC_PARAMETER_H_
