/*
 * Copyright (c) 2017 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <gtest/gtest.h>

#include <iostream>

#include "idlc/document.h"
#include "idlc/block.h"

class DocumentTest : public testing::Test {
 public:
  tidl::Block* block;
  tidl::Document* document;

  virtual void SetUp() {
    document = new tidl::Document();
    block = new tidl::Block("TestBlock", tidl::Block::TYPE_INTERFACE,
        "", __LINE__);
  }
  virtual void TearDown() {
  }
};

TEST_F(DocumentTest, Document_Constructor) {
  tidl::Document *doc = new tidl::Document();
  EXPECT_NE(doc, nullptr);
  delete doc;
}

TEST_F(DocumentTest, Document_AddBlock) {
  document->AddBlock(block);
  bool flag = false;
  for (auto& blk : document->GetBlocks()) {
    if (blk->GetID() == "TestBlock")
      flag = true;
  }
  EXPECT_NE(flag, false);
}

TEST_F(DocumentTest, Document_GetBlocks) {
  document->AddBlock(block);
  bool flag = false;
  for (auto& blk : document->GetBlocks()) {
    if (blk->GetType() == tidl::Block::TYPE_INTERFACE)
      flag = true;
  }
  EXPECT_NE(flag, false);
}

TEST_F(DocumentTest, Document_ExistBlock) {
  document->AddBlock(block);
  tidl::Block* testBlock = new tidl::Block("TestBlock",
      tidl::Block::TYPE_INTERFACE, "", __LINE__);
  EXPECT_NE(testBlock, nullptr);
  EXPECT_EQ(document->ExistBlock(testBlock), true);
  delete testBlock;
}
