#!/bin/bash

#--------------------------------------------------------#
# Tizen Interface Definition Langauge Compiler Unit Test #
#--------------------------------------------------------#

PATH=/bin:/usr/bin:/sbin:/usr/sbin

SCRIPT_FILE=$(readlink -f $0)
SCRIPT_DIR=$(dirname $SCRIPT_FILE)
BUILD_DIR=build

usage() {
	echo "Usage: $0 [command]"
	echo "Commands:"
	echo "    build              Build the unit tests"
	echo "    clean              Clean all artifacts"
	echo "    run                Run the unit tests"
	echo "    full               Build & Run the unit tests"
}

cmd_build() {
	echo "[TIDL Unit Test] Build Unit Tests"
	mkdir -p $SCRIPT_DIR/$BUILD_DIR
	cd $SCRIPT_DIR/$BUILD_DIR
	cmake ..
	make clean
	make
}

cmd_clean() {
	echo "[TIDL Unit Test] Clean all artifacts"
	rm -rf $SCRIPT_DIR/$BUILD_DIR
}

cmd_run() {
	echo "[TIDL Unit Test] Run Unit Tests"
	cd $SCRIPT_DIR/$BUILD_DIR
	$SCRIPT_DIR/$BUILD_DIR/tidl-unit-tests $SCRIPT_DIR/test.tidl $SCRIPT_DIR/test_error.tidl
	cd $SCRIPT_DIR
}

cmd_full() {
	cmd_build
	cmd_run
	cmd_clean
}

cmd=$1; shift;
case "$cmd" in
	build|--build|-b) cmd_build $@ ;;
	clean|--clean|-c) cmd_clean $@ ;;
	run|--run|-r) cmd_run $@ ;;
	full |--full |-f)  cmd_full $@ ;;
	*)     usage ;;

esac
